<?php
/**
 * Plugin Name: Consumer Financing by ChargeAfter
 * Plugin URI: https://docs.chargeafter.com/docs/woocommerce
 * Description: ChargeAfter connects merchants and lenders to offer consumers approved financing from multiple lenders across the entire credit spectrum.
 * Author: ChargeAfter
 * Author URI: http://www.chargeafter.com/
 * Version: 1.2.1
 * Requires at least: 4.8
 * Tested up to: 6.2
 *
 * WC requires at least: 3.0
 * WC tested up to: 7.7
 *
 * Text Domain: chargeafter-extension
 *
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! defined('CHARGE_AFTER_PLUGIN_URL')) {
	define('CHARGE_AFTER_PLUGIN_NAME', dirname( plugin_basename( __FILE__ ) ));
}

if ( ! class_exists( 'WC_ChargeAfter' ) ) :

	/**
	 * Main ChargeAfter Class
	 */
	class WC_ChargeAfter {

		/**
		 * WC_ChargeAfter Instance
		 *
		 * @var WC_ChargeAfter
		 */
		private static $instance;

		/**
		 * Get Instance
		 *
		 * @return WC_ChargeAfter
		 */
		public static function get_instance() {
			if ( null === self::$instance ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		/**
		 * WC_ChargeAfter constructor.
		 */
		public function __construct() {
			add_action( 'plugins_loaded', array( $this, 'init' ) );
		}

		/**
		 * Init plugin
		 */
		public function init() {
			self::requirePackages();

			add_filter( 'woocommerce_payment_gateways', array( $this, 'add_gateways' ) );
			add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'plugin_action_links' ) );
		}

		/**
		 * Require plugin packages
		 *
		 * @return boolean
		 */
		public static function requirePackages() {
			$root_include_path = dirname( __FILE__ ) . '/includes';

			require_once( $root_include_path . '/wc-chargeafter-charge-state.php' );
			require_once( $root_include_path . '/wc-chargeafter-helper.php' );
			require_once( $root_include_path . '/wc-chargeafter-api.php' );
			require_once( $root_include_path . '/wc-gateway-chargeafter.php' );
			require_once( $root_include_path . '/wc-chargeafter-order-handler.php' );
			require_once( $root_include_path . '/wc-chargeafter-widget.php' );
			require_once( $root_include_path . '/wc-chargeafter-start-checkout-script.php' );
			require_once( $root_include_path . '/wc-chargeafter-product-promo.php' );
			require_once( $root_include_path . '/wc-chargeafter-product-meta.php' );

			return true;
		}

		/**
		 * Add Gateway
		 *
		 * @param $methods
		 *
		 * @return array
		 */
		public function add_gateways( $methods ) {
			if ( WC_ChargeAfter_Helper::is_allow_gateway() ) {
				$methods[] = 'WC_ChargeAfter_Gateway';
			}

			return $methods;
		}

		/**
		 * Add action links to Woocommerce Payment`s
		 *
		 * @param $links
		 *
		 * @return array
		 */
		public function plugin_action_links( $links ) {
			$plugin_links = array(
				'<a href="admin.php?page=wc-settings&tab=checkout&section=wc_chargeafter_gateway">' .
				esc_html__( 'Settings', 'chargeafter-extension' ) .
				'</a>',
			);

			return array_merge( $plugin_links, $links );
		}
	}

	/**
	 * Init WC_ChargeAfter class
	 * Check if WooCommerce is active
	 */
	if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
		WC_ChargeAfter::get_instance();
	}

endif;
