<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class WC_ChargeAfter_Product_Meta
 */
class WC_ChargeAfter_Product_Meta {
	/**
	 * WC_ChargeAfter_Product_Meta Instance
	 *
	 * @var WC_ChargeAfter
	 */
	private static $instance;

	/**
	 * WC_ChargeAfter_Product_Meta construct
	 */
	public function __construct() {
		if ( WC_ChargeAfter_Helper::is_enabled() ) {
			add_action( 'woocommerce_product_options_general_product_data', array(
				$this,
				'add_option_group'
			) );

			add_action( 'woocommerce_process_product_meta', array( $this, 'save_option_field' ) );

			add_action( 'woocommerce_product_bulk_edit_start', array( $this, 'add_bulk_edit_group' ) );
			add_action( 'woocommerce_product_bulk_edit_save', array( $this, 'save_bulk_edit_field' ) );
		}
	}

	/**
	 * Get Instance
	 *
	 * @return WC_ChargeAfter
	 */
	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Echo custom option
	 *
	 * @return void
	 */
	public function add_option_group() {
		?>
			<div class="option_group">
				<?php
					$option_data = $this->get_meta_post_option_data( 'non-leasable' );
					woocommerce_wp_checkbox(
						array(
							'id' => 'wc_chargeafter_non-leasable',
							'value' => get_post_meta( get_the_ID(), 'wc_chargeafter_non-leasable', true),
							'label' => $option_data['label'],
							'desc_tip' => true,
							'description' => $option_data['description']
						)
					);
				?>
			</div>
		<?php
	}

	/**
	 * Save custom option
	 *
	 * @param $id
	 *
	 * @return void
	 */
	public function save_option_field( $id ) {
		$this->update_non_leasable_meta( $id, $_POST[ 'wc_chargeafter_non-leasable' ] );
	}

	/**
	 * Update post meta value
	 *
	 * @param $id
	 * @param $value
	 *
	 * @return void
	 */
	public function update_non_leasable_meta( $id, $value ) {
		$non_leasable = isset( $value ) && 'yes' === $value ? 'yes' : 'no';
		update_post_meta( $id, 'wc_chargeafter_non-leasable', $non_leasable );
	}

	/**
	 * Add bulk edit action
	 *
	 * @return void
	 */
	public function add_bulk_edit_group() {
		$option_data = $this->get_meta_post_option_data( 'non-leasable' );

		?>
			<div class="inline-edit-group">
				<label for="wc_chargeafter_non-leasable">
					<span class="title">
						<?php echo $option_data['label']; ?>
					</span>
					<span class="input-text-wrap">
						<select id="wc_chargeafter_non-leasable" name="wc_chargeafter_non-leasable">
							<?php
								$options = array(
									''    => __( '— No change —', 'woocommerce' ),
									'yes' => __( 'Yes', 'woocommerce' ),
									'no'  => __( 'No', 'woocommerce' ),
								);

								foreach ( $options as $key => $value ) {
									echo '<option value="' . esc_attr( $key ) . '">' . esc_html( $value ) . '</option>';
								}
							?>
						</select>
						<?php echo wc_help_tip( $option_data['description'] ); ?>
					</span>
				</label>
			</div>
		<?php
	}

	/**
	 * Save bulk edit option
	 *
	 * @param $product
	 *
	 * @return void
	 */
	public function save_bulk_edit_field( $product ) {
		$post_id = $product->get_id();

		if ( isset( $_REQUEST['wc_chargeafter_non-leasable'] ) ) {
			$this->update_non_leasable_meta( $post_id, $_REQUEST['wc_chargeafter_non-leasable'] );
		}
	}

	private function get_meta_post_option_data( $id ) {
		$data = [
			'non-leasable' => [
				'label' => __( 'Non Leasable', 'chargeafter-extension' ),
				'description' => sprintf(
					'%s %s',
					__( 'Specifying whether a product is leasable for', 'chargeafter-extension'),
					__( 'Consumer Financing by ChargeAfter', 'chargeafter-extension' )
				)
			]
		];

		return key_exists( $id, $data ) ? $data[$id] : null;
	}
}

/**
 * Init WC_ChargeAfter_Order_Handler
 */
WC_ChargeAfter_Product_Meta::get_instance();
