<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class WC_ChargeAfter_Widget
 */
class WC_ChargeAfter_Product_Promo {
	/**
	 * WC_ChargeAfter Instance
	 *
	 * @var WC_ChargeAfter
	 */
	private static $instance;

	/**
	 * WC_ChargeAfter_Product_Promo constructor.
	 */
	public function __construct() {
		if (
				WC_ChargeAfter_Helper::is_enabled() &&
				WC_ChargeAfter_Helper::is_enabled_promo()
		) {
			add_action( 'woocommerce_single_product_summary', array( $this, 'init' ), 5 );
			add_shortcode( 'chargeafter-promo-product', array( $this, 'init' ) );
		}
	}

	/**
	 * Get Instance
	 *
	 * @return WC_ChargeAfter
	 */
	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Init Promo
	 */
	public function init() {
		try {
			global $product;
			if (empty($product)) {
				throw new Exception('Empty product variable');
			}

			$widget_type = WC_ChargeAfter_Helper::get_promo_type();
			$ca_page_url   = WC_ChargeAfter_Helper::get_promo_url();

			try {
				$product_sku = $product->get_sku();
			} catch (Exception $exception) {
				$product_sku = '';
			}

			try {
				$product_price = $product->get_price();
			} catch (Exception $exception) {
				$product_price = null;
			}

			try {
				$not_allow_product_type = (
					$product->is_type( 'external' ) || $product->is_type( 'grouped' )
				);
			} catch (Exception $exception) {
				$not_allow_product_type = true;
			}

			if ( $product_price <= CHARGEAFTER_PROMO_MIN_PRICE || $not_allow_product_type || empty($product_price) ) {
				throw new Exception('Used not all params');
			}

			WC_ChargeAfter_Widget::initPromoScript();
		} catch (Exception $exception) {
			return;
		}

		?>
		<script>
			var caConfig = {
				apiKey: "<?php echo esc_html(WC_ChargeAfter_Helper::get_public_key()); ?>",
				url: "<?php echo esc_html(WC_ChargeAfter_Helper::get_url_suffix()); ?>"
			}
		</script>
		<div class="ca-promotional-widget ca-product"
			 data-widget-type="<?php echo esc_html($widget_type); ?>"
			 data-widget-item-sku="<?php echo esc_html($product_sku); ?>"
			 data-widget-item-price="<?php echo esc_html($product_price); ?>"
			 data-widget-financing-page-url="<?php echo esc_url($ca_page_url); ?>"
			 style="margin-bottom: 10px"
		></div>
		<?php
	}
}

/**
 * Init WC_ChargeAfter class
 */
WC_ChargeAfter_Product_Promo::get_instance();
