<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * ChargeAfter Gateway ID
 */
const CHARGEAFTER_ID = 'wc_chargeafter_gateway';

/**
 * ChargeAfter Promo Min Price
 */
const CHARGEAFTER_PROMO_MIN_PRICE = 0;

/**
 * Class WC_ChargeAfter_Gateway Extends WC_Payment_Gateway
 * ChargeAfter Gateway
 */
class WC_ChargeAfter_Gateway extends WC_Payment_Gateway {
	/**
	 * WC_ChargeAfter_Gateway constructor.
	 */
	public function __construct() {
		/*
		 * Payment Data
		 */
		$this->id                 = CHARGEAFTER_ID;
		$this->has_fields         = false;
		$this->method_title       = __( 'Consumer Financing by ChargeAfter', 'chargeafter-extension' );
		$this->method_description = __( 'ChargeAfter is a market-leading platform that connects retailers and lenders to offer consumers personalized Point of Sale Financing options.', 'chargeafter-extension' );

		/*
		 * Init forms
		 */
		$this->init_form_fields();
		$this->init_settings();

		/*
		 * Payment Data
		 */
		$this->enabled           = $this->get_option( 'enabled' );
		$this->title             = $this->get_option( 'title' );
		$this->description       = null;
		$this->order_button_text = __( 'Continue to payment' );
		$this->supports          = array( 'refunds' );

		$this->icon				 = null;

		/*
		 * Actions
		 */
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array(
				$this,
				'process_admin_options'
		) );

		add_action( 'woocommerce_before_checkout_form', array( $this, 'before_checkout_form' ) );
		add_action( 'woocommerce_api_wc_chargeafter_get_checkout_data', array(
				$this,
				'get_checkout_data_api_callback'
		) );
		add_action( 'woocommerce_api_wc_chargeafter_create_charge', array( $this, 'create_charge' ) );
		add_action( 'woocommerce_api_wc_chargeafter_on_data_update', array( $this, 'on_data_update' ) );
		add_action( 'woocommerce_api_wc_chargeafter_on_error_url', array( $this, 'on_error_url' ) );
		add_action( 'woocommerce_api_wc_chargeafter_on_error_callback_url', array( $this, 'on_error_callback_url' ) );
		add_action( 'update_option_woocommerce_' . CHARGEAFTER_ID . '_settings', array(
				$this,
				'update_option_hook'
		), 12, 3 );
		/*
		 * Filters
		 */
		add_filter( 'woocommerce_payment_complete_order_status', array(
				$this,
				'change_payment_complete_order_status'
		), 10, 2 );
		add_filter( 'woocommerce_available_payment_gateways', array(
				$this,
				'change_payment_gateway_disable_country'
		) );
	}

	/**
	 * Init Form Settings
	 */
	public function init_form_fields() {
		$this->form_fields = require( dirname( __FILE__ ) . '/admin/chargeafter-settings.php' );
	}

	/**
	 * Output the admin options table.
	 */
	public function admin_options() {
		?>
		<table class="form-table"><?php $this->generate_settings_html( $this->get_form_fields(), true ); // WPCS: XSS ok. ?></table>
		<script>
			function toogleFieldDisabled(e) {
				let d = document.getElementById("<?php echo esc_html('woocommerce_' . CHARGEAFTER_ID . '_'); ?>" + e);
				d.toggleAttribute("readonly");
			}
		</script>
		<?php
	}

	/**
	 * Override Settings Html
	 *
	 * @param array $form_fields
	 * @param bool $echo
	 *
	 * @return string
	 */
	public function generate_settings_html( $form_fields = array(), $echo = true ) {
		if ( empty( $form_fields ) ) {
			$form_fields = $this->get_form_fields();
		}

		foreach ( $form_fields as $key => $field ) {
			if ( isset( $field['parent_element'] ) ) {
				foreach ( $field['parent_element'] as $setting => $value ) {
					$setting_param = WC_ChargeAfter_Helper::get_setting_param( $setting );
					if ( ! is_null( $setting_param ) && $setting_param != $value ) {
						$form_fields[ $key ]['custom_attributes']['readonly'] = 'readonly';
					}
				}
			}
		}

		return parent::generate_settings_html( $form_fields, $echo );
	}

	/**
	 * Generate Radio Buttons
	 *
	 * @param $key
	 * @param $data
	 *
	 * @return false|string
	 */
	public function generate_radio_image_html( $key, $data ) {
		$field_key = $this->get_field_key( $key );
		$defaults  = array(
				'title'             => '',
				'label'             => '',
				'disabled'          => false,
				'class'             => '',
				'css'               => '',
				'type'              => 'text',
				'desc_tip'          => false,
				'custom_attributes' => array(),
		);

		$data = wp_parse_args( $data, $defaults );
		if ( ! $data['label'] ) {
			$data['label'] = $data['title'];
		}

		ob_start();
		?>
		<tr valign="top">
			<th scope="row" class="titledesc">
				<label for="<?php echo esc_attr( $field_key ); ?>">
					<?php
						echo wp_kses_post( $data['title'] );
						echo wp_kses_post( $this->get_tooltip_html( $data ) ); // WPCS: XSS ok.
					?>
				</label>
			</th>
			<td class="forminp radio-image <?php echo esc_attr( $data['class'] ); ?>">
				<fieldset id="<?php echo esc_attr( $field_key ); ?>">
					<legend class="screen-reader-text"><span><?php echo wp_kses_post( $data['title'] ); ?></span>
					</legend>

					<div class="label-wrap">
						<?php foreach ( $data['options'] as $datum => $title ) : ?>
							<div class="label-wrap-item">
								<label>
									<input name="<?php echo esc_attr($field_key); ?>" type="radio"
										   id="<?php echo esc_attr($datum); ?>" value="<?php echo esc_html($datum); ?>"
											<?php checked( WC_ChargeAfter_Helper::checkout_brand_deprecated($this->get_option( $key )), $datum ); ?>
											<?php echo wp_kses_post($this->get_custom_attribute_html( $data )); // WPCS: XSS ok. ?>
									/>

									<img src="<?php echo esc_attr(WC_ChargeAfter_Helper::get_checkout_brand_with_cdn($datum)); ?>"
										 alt="<?php echo esc_attr( $title ); ?>">
								</label>
							</div>
						<?php endforeach; ?>
					</div>
				</fieldset>
				<style>.radio-image .label-wrap,.radio-image label{display:flex!important;align-items:center}.radio-image .label-wrap{max-width:600px;flex-wrap:wrap}.radio-image .label-wrap .label-wrap-item{margin:0 10px 10px 0;flex:calc(50% - 10px)}.radio-image label input{margin:0 10px 0 0!important}.radio-image label img{max-width:198px}</style>
			</td>
		</tr>
		<?php

		return ob_get_clean();
	}

	/**
	 * Generate Hidden Input
	 *
	 * @param $key
	 * @param $data
	 *
	 * @return false|string
	 */
	public function generate_hidden_html( $key, $data ) {
		$field_key = $this->get_field_key( $key );
		$defaults  = array(
				'title'             => '',
				'disabled'          => false,
				'class'             => '',
				'css'               => '',
				'placeholder'       => '',
				'type'              => 'hidden',
				'desc_tip'          => false,
				'description'       => '',
				'custom_attributes' => array(),
		);

		$data = wp_parse_args( $data, $defaults );

		ob_start();
		?>
		<tr valign="top" style="display: none">
			<th scope="row" class="titledesc"></th>
			<td class="forminp">
				<fieldset>
					<input
							class="input-text regular-input <?php echo esc_attr( $data['class'] ); ?>"
							type="<?php echo esc_attr( $data['type'] ); ?>"
							name="<?php echo esc_attr( $field_key ); ?>"
							id="<?php echo esc_attr( $field_key ); ?>"
							style="<?php echo esc_attr( $data['css'] ); ?>"
							value="<?php echo esc_attr( $this->get_option( $key ) ); ?>"
							placeholder="<?php echo esc_attr( $data['placeholder'] ); ?>"
							<?php disabled( $data['disabled'], true ); ?>
							<?php echo esc_attr( $this->get_custom_attribute_html( $data ) ); // WPCS: XSS ok. ?> />
				</fieldset>
			</td>
		</tr>
		<?php

		return ob_get_clean();
	}

	/**
	 * Get option
	 *
	 * @param string $key
	 * @param null $empty_value
	 *
	 * @return string
	 */
	public function get_option( $key, $empty_value = null ) {
		return parent::get_option($key, $empty_value);
	}

	/**
	 * Update option
	 *
	 * @param string $key
	 * @param string $value
	 * @return bool
	 */
	public function update_option( $key, $value = '' ) {
		if ('icon' == $key) {
			$value = WC_ChargeAfter_Helper::get_merchant_brand_id($value);
		}

		return parent::update_option($key, $value);
	}

	/**
	 * Hook update_option_woocommerce_wc_chargeafter_gateway_settings
	 *
	 * @param $old_value
	 * @param $value
	 * @param $option
	 */
	public function update_option_hook( $old_value, $value, $option ) {
		$old_icon = key_exists('icon', $old_value) ? $old_value['icon'] : '';
		$this->update_option('icon', key_exists('icon', $value) ? $value['icon'] : $old_icon);
	}

	/**
	 * Get Checkout Api Data
	 */
	public function get_checkout_data_api_callback() {
		$nonce    = ( isset($_GET['nonce']) ? sanitize_text_field($_GET['nonce']) : null );
		$order_id = ( isset($_GET['orderId']) ? sanitize_text_field($_GET['orderId']) : null );

		if ( wp_verify_nonce( $nonce, $this->nonce_key( $order_id ) ) ) {
			status_header( 200 );

			$order = wc_get_order( $order_id );
			echo wp_kses_post(WC_ChargeAfter_Start_Checkout_Script::checkout_data( $order ));
		} else {
			status_header( 401 );
		}

		exit;
	}

	/**
	 * Get nonce key
	 *
	 * @param $order_id
	 *
	 * @return string
	 */
	private function nonce_key( $order_id ) {
		return 'chargeafter-extension_' . $order_id;
	}

	/**
	 * Action before checkout form
	 *
	 */
	public function before_checkout_form() {
		print_r(WC_ChargeAfter_Start_Checkout_Script::start_checkout_script(
			WC_ChargeAfter_Helper::get_public_key(), WC_ChargeAfter_Helper::get_url_suffix()
		));
	}

	/**
	 * Filter woocommerce payment complete
	 *
	 * @param $status
	 * @param int $order_id
	 *
	 * @return string
	 */
	public function change_payment_complete_order_status( $status, $order_id = 0 ) {
		if ( WC_ChargeAfter_Helper::is_chargeafter_gateway( $order_id ) ) {
			$status = 'on-hold';
		}

		return $status;
	}

	/**
	 * Filter woocommerce payment available
	 *
	 * @param $available_gateways
	 *
	 * @return mixed
	 */
	public function change_payment_gateway_disable_country( $available_gateways ) {
		if ( is_checkout() && key_exists( CHARGEAFTER_ID, $available_gateways ) ) {
			$available_gateways[ CHARGEAFTER_ID ]->title = $this->prepareBrandImg();
		}

		return $available_gateways;
	}

	/**
	 * Prepare img content
	 *
	 * @return string
	 */
	private function prepareBrandImg() {
		$checkout_brand = WC_ChargeAfter_Helper::get_checkout_brand();
		$checkout_img	= WC_ChargeAfter_Helper::get_checkout_brand_with_cdn( $checkout_brand );

		return "<img src='{$checkout_img}' alt='' style='float:none' >";
	}

	/**
	 * Override Process Payment.
	 *
	 * @param int $order_id
	 *
	 * @return array
	 */
	public function process_payment( $order_id ) {
		update_post_meta( $order_id, '_payment_method_title', sanitize_text_field( $this->get_option( 'title' )) );

		$hash = '#checkoutfinance=1;orderid=' . $order_id . ';nonce=' . wp_create_nonce( $this->nonce_key( $order_id ) );

		return array(
				'result'   => 'success',
				'redirect' => wc_get_checkout_url() . $hash
		);
	}

	/**
	 * Action Create ChargeAfter
	 */
	public function create_charge() {
		$nonce    = ( isset($_GET['nonce']) ? sanitize_text_field($_GET['nonce']) : null );
		$order_id = ( isset($_GET['orderId']) ? sanitize_text_field($_GET['orderId']) : null );

		if ( wp_verify_nonce( $nonce, $this->nonce_key( $order_id ) ) ) {

			$token       = ( isset($_GET['confirmationToken']) ? sanitize_text_field($_GET['confirmationToken']) : null );
			$charge_data = WC_ChargeAfter_API::api_create_charge( $token );
			$order       = wc_get_order( $order_id );

			if ( $charge_data ) {

				global $woocommerce;

				$woocommerce->cart->empty_cart();
				$lender_name = $charge_data['offer']['lender']['name'];

				if ( is_string( $lender_name ) && null != $lender_name ) {
					$order->add_order_note( "Order created via ChargeAfter by {$lender_name}");
				}

				$order->payment_complete( $charge_data['id'] );

				WC_ChargeAfter_Helper::add_new_auth_note( $order_id );
				WC_ChargeAfter_API::api_payment_partially_update( $charge_data['id'], $order_id );

				if (WC_ChargeAfter_Helper::is_enabled_auto_settle()) {
					$order->update_status('completed');
				}

				echo esc_url_raw($this->get_return_url( $order ));
			} else {
				$order->update_status( 'pending', __( 'Pending payment', 'chargeafter-extension' ) );

				status_header( 424 );
				wc_add_notice( 'Failed to finalize payment', 'error' );

				echo esc_url_raw(wc_get_checkout_url());
			}
		} else {
			status_header( 401 );
			wc_add_notice( 'Failed to create checkout', 'error' );

			echo esc_url_raw(wc_get_checkout_url());
		}

		exit;
	}

	/**
	 * Update Callback
	 */
	public function on_data_update() {
		$nonce    = ( isset($_GET['nonce']) ? sanitize_text_field($_GET['nonce']) : null );
		$order_id = ( isset($_GET['orderId']) ? sanitize_text_field($_GET['orderId']) : null );

		if ( wp_verify_nonce( $nonce, $this->nonce_key( $order_id ) ) ) {
			$consumer_details = ( isset($_POST['consumerDetails']) ? WC_ChargeAfter_Helper::recursive_sanitize_text_field($_POST, 'consumerDetails') : array() );
			$order            = wc_get_order( $order_id );

			$billing_address = array(
				'first_name' => $consumer_details['firstName'],
				'last_name'  => $consumer_details['lastName'],
				'address_1'  => $consumer_details['billingAddress']['line1'],
				'address_2'  => $consumer_details['billingAddress']['line2'],
				'city'       => $consumer_details['billingAddress']['city'],
				'state'      => $consumer_details['billingAddress']['state'],
				'postcode'   => $consumer_details['billingAddress']['zipCode'],
				'country'    => 'US',
				'email'      => $consumer_details['email'],
				'phone'      => $consumer_details['phoneNumber'],
			);
			$order->set_address( $billing_address, 'billing' );

			$shipping_address = array(
				'first_name' => $consumer_details['firstName'],
				'last_name'  => $consumer_details['lastName'],
				'address_1'  => $consumer_details['shippingAddress']['line1'],
				'address_2'  => $consumer_details['shippingAddress']['line2'],
				'city'       => $consumer_details['shippingAddress']['city'],
				'state'      => $consumer_details['shippingAddress']['state'],
				'postcode'   => $consumer_details['shippingAddress']['zipCode'],
				'country'    => 'US',
				'email'      => $consumer_details['email'],
				'phone'      => $consumer_details['phoneNumber'],
			);
			$order->set_address( $shipping_address, 'shipping' );

			$order->calculate_totals();
			$new_amounts = array(
				'taxAmount'      => floatval( $order->get_total_tax() ),
				'shippingAmount' => floatval( $order->get_shipping_total() ),
				'totalAmount'    => floatval( $order->get_total() ),
			);

			status_header( 200 );

			echo json_encode( $new_amounts );
		} else {
			status_header( 401 );
		}
		exit;
	}

	/**
	 * Error url Callback
	 */
	public function on_error_url() {
		$error      = ( isset($_GET['errorMsg']) ? sanitize_text_field($_GET['errorMsg']) : null );
		$error_code = ( isset($_GET['errorCode']) ? sanitize_text_field($_GET['errorCode']) : null );

		if ( !in_array($error_code, ['BACK_TO_STORE', 'CONSUMER_CANCELLED']) ) {
			wc_add_notice( $this->title . ' - ' . $error, 'error' );
		}

		echo esc_url_raw(wc_get_checkout_url());
		exit;
	}

	/**
	 * Error url Callback
	 */
	public function on_error_callback_url() {
		$order_id = ( isset($_GET['orderId']) ? sanitize_text_field($_GET['orderId']) : null );

		if ( $order_id && WC_ChargeAfter_Helper::is_chargeafter_gateway($order_id) ) {
			$order = wc_get_order( $order_id );

			if ($order->get_status() == 'pending') {
				$order->delete();
			}
		}

		status_header( 204 );
		exit;
	}

	/**
	 * Override Process refund.
	 *
	 * @param int $order_id
	 * @param null $amount
	 * @param string $reason
	 *
	 * @return bool
	 */
	public function process_refund( $order_id, $amount = null, $reason = '' ) {
		return ( WC_ChargeAfter_Order_Handler::process_refund_amount( $order_id, $amount ) );
	}
}
